// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KDDOCKWIDGETS_VERSION_H
#define KDDOCKWIDGETS_VERSION_H

#define KDDOCKWIDGETS_VERSION_STRING "2.4.0"
#define KDDOCKWIDGETS_VERSION_MAJOR 2
#define KDDOCKWIDGETS_VERSION_MINOR 4
#define KDDOCKWIDGETS_VERSION_PATCH 0
#define KDDOCKWIDGETS_VERSION KDDOCKWIDGETS_VERSION_CHECK(KDDOCKWIDGETS_VERSION_MAJOR, KDDOCKWIDGETS_VERSION_MINOR, KDDOCKWIDGETS_VERSION_PATCH)

/*
   for example: KDDOCKWIDGETS_VERSION >= KDDOCKWIDGETS_VERSION_CHECK(1, 2, 2))
*/
#define KDDOCKWIDGETS_VERSION_CHECK(major, minor, patch) ((major<<16)|(minor<<8)|(patch))

#endif
